/*
 * ScenarioItem.java
 *
 * Created on 5  2004 ., 12:32
 */

package fi.helsinki.dacopan.animseq;

import fi.helsinki.dacopan.ui.MainFrame;

/**
 * An interface for all scenario items stored in the animation sequence. Each item should store
 * an individual copy (clone, if necessary) of the settings it needs. 
 *
 * <p><b>NOTE:</b> The implementing class itself is responsible for creating the necessary
 * clones of objects passed to it.
 */
public interface ScenarioItem {
    
    /** 
     * Performs the action stored in this scenario item.
     *
     * @param mainFrame A reference to a MainFrame instance in order to be able to carry out
     * necessary mode changes.
     * @param inRecordingMode Tells the item if the AnimationSequence is in recording mode or not.
     * If true, the item should refraing from any potentially confusing mode changes, like
     * automatically putting AnimationTimeState in play mode.
     */
    public void doAction(MainFrame mainFrame, boolean inRecordingMode);
    
    /**
     * Returns a short text representation of the scenario item for presentation in the
     * playlist.
     */
    public String getPlaylistText();
    
    /**
     * @return True if recording the end was successful, false if not. The animation sequence is
     * only allowed to move into recording the next item if the recording was successful.
     */
    public boolean recordEnd(float nowTime);
    
    /**
     * Should return true if the item will set the main frame to some reasonable state
     * and false otherwise. Examples of items that should return true are MSC and ENC
     * items, and examples of items returning false are pause item and end marker item.
     */
    public boolean getShowable();
    
}
