package fi.helsinki.dacopan.model;

/**
 * A flow (e.g. a TCP connection) in a scenario.
 * 
 * @version $Id: Flow.java,v 1.3 2004/05/07 09:19:42 aarniala-dacopan Exp $
 */
public class Flow implements Identifiable {
    
    /** unique flow id */
    protected long id;

    /** first endpoint */
    protected Host first;
    
    /** port of the first endpoint */
    protected int firstPort;
    
    /** second endpoint */
    protected Host second;
    
    /** port of the second endpoint */
    protected int secondPort;
    
    public Flow(long id, Host first, int firstPort, Host second, int secondPort) {
        if(first == null || second == null) {
            throw new IllegalArgumentException("Both hosts must be specified");
        }
        this.id = id;
        this.first = first;
        this.firstPort = firstPort;
        this.second = second;
        this.secondPort = secondPort;
    }
    
    public Host getFirst() {
        return first;
    }

    public int getFirstPort() {
        return firstPort;
    }

    public long getId() {
        return id;
    }

    public Host getSecond() {
        return second;
    }

    public int getSecondPort() {
        return secondPort;
    }
    
    public boolean equals(Object o) {
        if(this == o) return true;
        if(!(o instanceof Flow)) return false;
        Flow other = (Flow)o;
        if(other.id != id) return false;
        if(!other.first.equals(first)) return false;
        if(other.firstPort != firstPort) return false;
        if(!other.second.equals(second)) return false;
        if(other.secondPort != secondPort) return false;
        return true;
    }
    
    // TODO hashcode
    
    public String toString() {
        return "Flow {" + id + "/" + first + "/" + second + " }";
    }

}
