package fi.helsinki.dacopan.model;

import java.util.LinkedList;
import java.util.List;

/**
 * A network link between two {@link Host hosts}.
 * 
 * @version $Id: Link.java,v 1.2 2004/05/07 09:19:42 aarniala-dacopan Exp $
 */
public class Link implements Identifiable {

    /** unique link id */
    protected long id;
    
    /** first host */
    protected Host first;
    
    /** second host */
    protected Host second;
    
    /** list of static variables associated with this link */
    protected List staticVariables = new LinkedList();

    /**
     * Creates a link with the given values.
     */
    public Link(long id, Host first, Host second) {
        if(first == null || second == null) {
            throw new IllegalArgumentException("Both hosts must be non-null");
        }
        this.id = id;
        this.first = first;
        this.second = second;
    }
    
    /**
     * Creates a link with the given values and static variables.
     */
    public Link(long id, Host first, Host second, List staticVariables) {
        this(id, first, second);
        setStaticVariables(staticVariables);
    }
    
    public long getId() {
        return id;
    }

    public Host getFirst() {
        return first;
    }

    public Host getSecond() {
        return second;
    }
    
    /**
     * Returns a non-null list of static variables associated with this link.
     */
    public List getStaticVariables() {
        return staticVariables;
    }

    public void setStaticVariables(List staticVariables) {
        if(staticVariables == null) {
            throw new IllegalArgumentException("Static variables must be non-null");
        }
        this.staticVariables = staticVariables;
    }
    
    public boolean equals(Object o) {
        if(this == o) return true;
        if(!(o instanceof Link)) return false;
        Link other = (Link)o;
        if(other.id != id) return false;
        if(!other.first.equals(first)) return false;
        if(!other.second.equals(second)) return false;
        if(!other.staticVariables.equals(staticVariables)) return false;
        return true;
    }

    public String toString() {
        return "Link {" + id + "/F:" + first + "/S:" + second + "/" + staticVariables + "}";
    }
    
}
