package fi.helsinki.dacopan.model;

import fi.helsinki.dacopan.Localization;

/**
 * A variable with a static value specific to a host, link, protocol
 * or a combination of them.
 * 
 * @see fi.helsinki.dacopan.model.VariableDefinition
 * @version $Id: StaticVariable.java,v 1.4 2004/05/06 07:46:32 jbrown-dacopan Exp $
 */
public class StaticVariable {

    /** name of the variable */
    protected String name;
    
    /** value of the variable */
    protected String value;

    /** the host this variable is associated with, if any */
    protected Host host;
    
    /** the link this variable is associated with, if any */
    protected Link link;
    
    /** the protocol this variable is associated with, if any */
    protected Protocol protocol;
    
    /**
     * Creates a static variable with the given name and value (required).
     * One of host, link and protocol must also be specified.
     */
    public StaticVariable(String name, String value, Host host, Link link, Protocol protocol) {
        if(name == null || value == null) {
            throw new IllegalArgumentException("Name and value are required");
        }
        if(host == null && link == null && protocol == null) {
            throw new IllegalArgumentException("Host, link and protocol can't all be null");
        }
        this.name = name;
        this.value = value;
        this.host = host;
        this.link = link;
        this.protocol = protocol;
    }
    
    public String getName() {
        return Localization.getString("model.constant." + name + ".abbr");
    }

    public String getFullName() {
        return Localization.getString("model.constant." + name + ".full");
    }
    
    public String getValue() {
        return value;
    }

    public Host getHost() {
        return host;
    }

    public Link getLink() {
        return link;
    }

    public Protocol getProtocol() {
        return protocol;
    }
    
    public boolean equals(Object o) {
        if(this == o) return true;
        if(!(o instanceof StaticVariable)) return false;
        StaticVariable other = (StaticVariable)o;
        if(!other.name.equals(name)) return false;
        if(!other.value.equals(value)) return false;
        // note: we can't compare hosts, links nor protocols here, as that
        // would cause circular references (those classes refer to us in
        // their own equals() -methods)
        return true;
    }
    
    public String toString() {
        return "{" + name + "=" + value + "}";
    }

    
}
