package fi.helsinki.dacopan.ui;

import java.awt.SystemColor;

import javax.swing.JPanel;

import fi.helsinki.dacopan.contsig.ControlSignalsListener;

/**
 * An abstract super-class for all the animation panels. The super-class makes
 * sure the animation panel has a reference to the main frame of the UI, and
 * implements the <code>ControlSignalsListener</code> interface.
 */
public abstract class AbstractAnimationPanel
    extends JPanel
    implements ControlSignalsListener 
{
    
    protected final MainFrame mainFrame;
    
    // Constructor
    public AbstractAnimationPanel(MainFrame mainFrame) {
        this.mainFrame = mainFrame;
        setBackground(SystemColor.control);
        //setBackground(UserInterface.DACOPAN_COLOR_PANE_BG);
    }
    
    /**
     * The default implementation just calls <code>stepTo</code> with the given
     * <code>nowTime</code> parameter.
     *
     * @see fi.helsinki.dacopan.contsig.ControlSignalsListener#advance(float, float)
     */
    public void advance(float step, float nowTime) {
        stepTo(nowTime);
    }
    
    /**
     * Abstract method that needs to be implemented by all animation panels.
     * This is the only method related to <code>ControlSignalsListener</code>
     * interface that doesn't have a default implementation.
     *
     * <p><b>Note:</b> It is the responsibility of the listening object to call
     * <code>repaint()</code> on itself as a result of receiving
     * this signal.
     *
     * @see fi.helsinki.dacopan.contsig.ControlSignalsListener#stepTo(float)
     */
    public abstract void stepTo(float nowTime);
    
    /**
     * Implemented by default as an empty method.
     */
    public void toPlayMode() {
        // default implementation is no-op
    }
    
    /**
     * Implemented by default as an empty method.
     */
    public void toPauseMode() {
        // default implementation is no-op
    }
}
