package fi.helsinki.dacopan.ui;

import java.io.File;

import javax.swing.filechooser.FileFilter;

/**
 * This class is used to set the file filter to loading 
 * and saving animation dialogs. 
 */
public class DaCoPAnFileFilter extends FileFilter {

    private String extension;
    private String description;

    /**
     * 
     * @param extension     The extension of the files that we want to allow
     * @param description   A short textual description to be presented in the UI
     */
    public DaCoPAnFileFilter(String extension, String description) {
        this.extension = extension;
        this.description = description;
    }

    private String getExtension(File file) {
        String ext = null;
        String s = file.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 && i < s.length() - 1) {
            ext = s.substring(i + 1).toLowerCase();
        }
        return ext;
    }

    /**
     * Checks if the given file is to be accepted. Overriden from FileFilter.
     */
    public boolean accept(File file) {
        if (file.isDirectory()) {
            return true;
        }

        String extension = getExtension(file);
        if (extension != null) {
            if (extension.equals(this.extension)) {
                return true;
            }
        }
        else {
            return false;
        }

        return false;
    }

    /**
     * Returns the description of the file type
     */
    public String getDescription() {
        return description + " (*." + extension + ")";
    }

}
