package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.model.Note;

/**
 * This dialog is used to edit and add notes
 */
public class EditNoteDialog extends JDialog {
    
    /**
     * This panel is used to edit and add notes
     */
    private class EditNotePanel extends JPanel {
                        
        // The needed widgets
        private JTextArea textArea;
        private JLabel timeLabel;
        private JTextField timeField;
        private JSpinner timeSpinner;
        private JButton saveButton;
        private JButton cancelButton;
        
        /** The note that the panel is editing */
        private Note note;
        
        /** The dialog that contains this panel */
        private JDialog parent;
        
        /** A reference to the note panel calling the dialog */
		protected NotePanel activeNotePanel = null;
        
        /** The number of rows for the text area */
		private static final int TEXT_AREA_ROWS = 8;
		
        /** The number of columns for the text area */
        private static final int TEXT_AREA_COLUMNS = 30;
        
        /**
         * 
         * @param parentDialog
         * @param mode
         * @param currentNote
         */
        public EditNotePanel(JDialog parentDialog, 
                NotePanel notePanel,
                int mode, Note currentNote) 
        {
            
            parent = parentDialog;
            note = currentNote;
            this.activeNotePanel = notePanel;
            
        	setLayout(new BorderLayout());
                        
            // Create the panel layout
                        
            textArea = new JTextArea(TEXT_AREA_ROWS,TEXT_AREA_COLUMNS);            
            textArea.setEditable(true);
            
            if(mode == NOTE_EDIT_MODE_EDIT) {
            	textArea.setText(currentNote.getText());
            }
            
            JScrollPane textScrollPane = new JScrollPane(textArea);
            textScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            textArea.setMargin(new Insets(5,10,5,25));
            textArea.setLineWrap(true);            
                                  
            JPanel buttonPanel = new JPanel(new GridBagLayout()); 
            
            GridBagConstraints c = new GridBagConstraints();                  
            c.gridx = 0;
            c.gridy = 0;
            saveButton = new JButton(Localization.getString("panel.edit_note.save"));
            UserInterface.setDefaultButtonFont(saveButton);
            
            if(mode == NOTE_EDIT_MODE_EDIT) {
                saveButton.addActionListener(new ActionListener(){    
                    public void actionPerformed(ActionEvent arg0) {
                    	note.setText(textArea.getText());
                        parent.hide();
                    }                    
                });
            }
            else {
                saveButton.addActionListener(new ActionListener(){    
                    public void actionPerformed(ActionEvent arg0) {
                        activeNotePanel.addNoteToCurrentPosition(textArea.getText());
                        parent.hide();
                    }                    
                });               
            }
            buttonPanel.add(saveButton, c);
            
            c.gridx = 1;
            cancelButton = new JButton(Localization.getString("panel.edit_note.cancel"));
            UserInterface.setDefaultButtonFont(cancelButton);
            
            cancelButton.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					parent.hide();
				}
                
            });
            buttonPanel.add(cancelButton, c);
                       
            add(textScrollPane, BorderLayout.CENTER);
            add(buttonPanel, BorderLayout.SOUTH);
        }        
    }
    
    /** The notes adding mode for using the edit dialog */
    public static final int NOTE_EDIT_MODE_ADD  = 0;
    
    /** The notes editing mode for using the edit dialog */
    public static final int NOTE_EDIT_MODE_EDIT = 1;
   
    /**
     * 
     * @param notePanel
     * @param mode
     * @param note
     */
    public EditNoteDialog(JFrame mainFrame, NotePanel notePanel, int mode, Note note) {
        super(mainFrame);
        if(mode == NOTE_EDIT_MODE_ADD) {
        	setTitle(Localization.getString("dialog.edit_note.title_add"));
        }
        else {
            setTitle(Localization.getString("dialog.edit_note.title_edit"));
        }
    	EditNotePanel panel = new EditNotePanel(this, notePanel,mode, note);
        getContentPane().add(panel);
    }    
}
