/*
 * SettingsMSC.java
 *
 * Created on 11. huhtikuuta 2004, 19:02
 */

package fi.helsinki.dacopan.ui;

import fi.helsinki.dacopan.model.Host;
import fi.helsinki.dacopan.model.Layer;
import fi.helsinki.dacopan.model.VariableDefinition;

/**
 *
 * @author  Vesa
 */
public class SettingsMSC 
    implements Cloneable
{
    
    
    private Host leftHost;
    private Host rightHost;
    private Layer layer;
    
    public static final int DEFAULT_VISUAL_SCALE = 500;
    public static final int MAX_VISUAL_SCALE = 2000;
    public static final int MIN_VISUAL_SCALE = 20;
    public static final int VISUAL_SCALE_SLIDER_STEP = 10;
    
    public static final int MAX_SHOW_VARIABLES = 5;
    public static final int MAX_SHOW_HEADER_FIELDS = 7;
    
    public static final int DEFAULT_SCALE_MODE = 1;
    
    public static final int SCALE_MODE_SECONDS = 1;
    public static final int SCALE_MODE_DSECONDS = 2;
    public static final int SCALE_MODE_CSECONDS = 3;
    public static final int SCALE_MODE_MILLISECONDS = 4;
    
    public static final String SCALE_MODE_MILLISECONDS_UNIT = "ms";
    public static final String SCALE_MODE_SECONDS_UNIT = "s";
    public static final String SCALE_MODE_DSECONDS_UNIT = "0.1s";
    public static final String SCALE_MODE_CSECONDS_UNIT = "0.01s";

    public static final float DEFAULT_TIME_SCALE = 5.0f;

    public static final float MAX_TIME_SCALE = 50.0f;
    public static final float MIN_TIME_SCALE = 0.05f;

    /** 
     * ratio between the timeline stored in Protocol Events File and the physical clock time of 
     * the animation. 
     * Number of seconds that takes to the animation to show one second of network time
     * (Animation Time) 
     */
    protected float timeScale = DEFAULT_TIME_SCALE;

    
    /** 
     * Unit: pixels / second (of timestamp time, not animation time); 
     * value 150 means that 150 pixels represent 1 second. 
     */
    private int visualScale = DEFAULT_VISUAL_SCALE;
    
    private int scaleMode = DEFAULT_SCALE_MODE;
    
    /** 
     * If true starts scenario item in play mode, otherwise starts it in pause mode.
     */
    private boolean scenarioAutoPlay = false;
    
    /** 
     * An array of variables whose values will be plotted in the center of the animation. 
     */
    private VariableDefinition[] variablesCenter;
    
    /** An array of variables whose values will be plotted in columns. The order of variables in
     * the column is processed so that the value at variablesColumns[0] will always be CLOSEST to
     * drawing area on both left and right sides.
     */
    private VariableDefinition[] variablesColumns;
    
    private float startTime = -1.0f;
    private float endTime = -1.0f;
        
    /** Creates a new instance of SettingsMSC */
    public SettingsMSC(
        Host leftHost, 
        Host rightHost, 
        Layer layer, 
        VariableDefinition[] variablesCenter,
        VariableDefinition[] variablesColumns)
    {
        if (leftHost == null ||
            rightHost == null ||
            layer == null ||
            variablesCenter == null ||
            variablesColumns == null)
        {
            throw new IllegalArgumentException("Null parameters not accepted!");
        }
        
        this.leftHost = leftHost;
        this.rightHost = rightHost;
        this.layer = layer;
        this.variablesCenter = variablesCenter;
        this.variablesColumns = variablesColumns;
    }

    /** Creates a copy instance of SettingsMSC */
    public SettingsMSC(SettingsMSC settingsMSC){
        if (settingsMSC == null){
            throw new IllegalArgumentException("Null parameters not accepted!");
        }
        this.scenarioAutoPlay = settingsMSC.getScenarioAutoPlay();
        this.timeScale = settingsMSC.getTimeScale();
        this.scaleMode = settingsMSC.getScaleMode();
        this.leftHost = settingsMSC.getLeftHost();
        this.rightHost = settingsMSC.getRightHost();
        this.layer = settingsMSC.getLayer();
        this.variablesCenter = settingsMSC.getVariablesCenter();
        this.variablesColumns = settingsMSC.getVariablesColumns();
        this.visualScale = settingsMSC.getVisualScale();
    }
    
    /** 
     * Getter for property leftHost.
     * @return Value of property leftHost.
     */
    public fi.helsinki.dacopan.model.Host getLeftHost() {
        return leftHost;
    }    
    
    /** 
     * Getter for property rightHost.
     * @return Value of property rightHost.
     */
    public fi.helsinki.dacopan.model.Host getRightHost() {
        return rightHost;
    }
    
    /** 
     * Getter for property layer.
     * @return Value of property layer.
     */
    public fi.helsinki.dacopan.model.Layer getLayer() {
        return layer;
    }
    
    
    /** Returns the scale mode */
    public int getScaleMode(){
        return scaleMode;
    }

    /** Sets the scale mode */
    public void setScaleMode(int value){
        scaleMode = value;
    }
    
    /** returns the time scale for the animation */
    public float getTimeScale() {
        return this.timeScale;
    }

    /** Sets the time scale for the animation. Cannot be negative or zero. */
    public void setTimeScale(float timeScale) {
        if (timeScale <= 0.0f) {
            throw new IllegalArgumentException("Scale cannot be negative or zero!");
        }
        this.timeScale = timeScale;
    }

    
    /**
     * Getter for property visualScale.
     * @return Value of property visualScale.
     */
    public int getVisualScale() {
        return visualScale;
    }
    
    /** 
     * Setter for property visualScale.
     * @param visualScale New value of property visualScale.
     */
    public void setVisualScale(int visualScale) {
        if (visualScale <= 0f) {
            throw new IllegalArgumentException("Scale cannot be zero or negative!");
        }
        this.visualScale = visualScale;
    }
    
    /** 
     * Getter for property variablesCenter.
     * @return Value of property variablesCenter.
     */
    public fi.helsinki.dacopan.model.VariableDefinition[] getVariablesCenter() {
        return this.variablesCenter;
    }
    
    /** 
     * Setter for property variablesCenter.
     * @param variablesCenter New value of property variablesCenter.
     */
    public void setVariablesCenter(fi.helsinki.dacopan.model.VariableDefinition[] variablesCenter) {
        this.variablesCenter = variablesCenter;
    }
    
    /** 
     * Getter for property variablesColumns.
     * @return Value of property variablesColumns.
     */
    public fi.helsinki.dacopan.model.VariableDefinition[] getVariablesColumns() {
        return this.variablesColumns;
    }
    
    /** 
     * Setter for property variablesColumns.
     * @param variablesColumns New value of property variablesColumns.
     */
    public void setVariablesColumns(fi.helsinki.dacopan.model.VariableDefinition[] variablesColumns) {
        this.variablesColumns = variablesColumns;
    }
    
    /** 
     * Getter for property startTime.
     * @return Value of property startTime.
     */
    public float getStartTime() {
        return this.startTime;
    }
    
    /** 
     * Setter for property startTime.
     * @param startTime New value of property startTime.
     */
    public void setStartTime(float newStartTime) {
        if (newStartTime < 0f) {
            throw new IllegalArgumentException("Start time cannot be negative!");
        }
        
        if (this.startTime >= 0f && 
            this.endTime > 0f && 
            newStartTime > this.endTime)
        {
            throw new IllegalArgumentException("Cannot change start time to be after end time!");
        }
        
        this.startTime = newStartTime;
    }
    
    /** 
     * Getter for property endTime.
     * @return Value of property endTime.
     */
    public float getEndTime() {
        return this.endTime;
    }
    
    /**
     * @param value
     */
    public void setScenarioAutoPlay(boolean value){
        this.scenarioAutoPlay = value;
    }

    /**
     * @return
     */
    public boolean getScenarioAutoPlay(){
        return this.scenarioAutoPlay;
    }
    
    /**
     * @return
     */
    public int getScaleMultiplier(){
        return getScaleMultiplier(scaleMode);
    }

    /**
     * @param mode
     * @return
     */
    public static int getScaleMultiplier(int mode){
        switch (mode) {
            case SCALE_MODE_SECONDS: return 1;
            case SCALE_MODE_DSECONDS: return 10;
            case SCALE_MODE_CSECONDS: return 100;
            case SCALE_MODE_MILLISECONDS: return 1000;
        }
        
        throw new RuntimeException("Illegal scale mode!");
    }

    /**
     * @return
     */
    public String getScaleMultiplierUnit(){
        return getScaleMultiplierUnit(scaleMode);
    }

    /**
     * @param mode
     * @return
     */
    public static String getScaleMultiplierUnit(int mode){
        switch (mode) {
            case SCALE_MODE_SECONDS: return SCALE_MODE_SECONDS_UNIT;
            case SCALE_MODE_CSECONDS: return SCALE_MODE_CSECONDS_UNIT;
            case SCALE_MODE_DSECONDS: return SCALE_MODE_DSECONDS_UNIT;
            case SCALE_MODE_MILLISECONDS: return SCALE_MODE_MILLISECONDS_UNIT;
        }
        
        throw new RuntimeException("Illegal scale mode!");
    }
    
    /** 
     * Setter for property endTime.
     * @param endTime New value of property endTime.
     */
    public void setEndTime(float newEndTime) {
        if (this.startTime < 0f) {
            throw new IllegalStateException("Must set start time before end time!");
        }
        
        if (newEndTime <= 0f) {
            throw new IllegalArgumentException("End time cannot be zero or negative!");
        }

        if (newEndTime <= this.startTime) {
            throw new IllegalArgumentException("Cannot change end time to be before start time!");
        }

        this.endTime = newEndTime;
    }

    /** Returns a true clone of this SettingsMSC object. */
    public Object clone() {
        return new SettingsMSC(this);
    }    
    
    public void setAllFields(SettingsMSC newSettings) {
        this.leftHost = newSettings.leftHost;
        this.rightHost = newSettings.rightHost;
        this.layer = newSettings.layer;
        this.timeScale = newSettings.timeScale;
        this.visualScale = newSettings.visualScale;
        this.scaleMode = newSettings.scaleMode;
        this.scenarioAutoPlay = newSettings.scenarioAutoPlay;
        this.variablesCenter = newSettings.variablesCenter;
        this.variablesColumns = newSettings.variablesColumns;
        this.startTime = newSettings.startTime;
        this.endTime = newSettings.endTime;
    }
    
}
