package fi.helsinki.dacopan.ui;

import java.util.Iterator;

import javax.swing.JRadioButton;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.model.Layer;
import fi.helsinki.dacopan.settings.GeneralSettings;


/**
 * <p>
 * JPanel containing the settings that can be changed to adjust the speed and scale to represent the
 * animation
 * </p>
 * <p>
 * This scale settings are changed in different way depending of the animator working mode. <br>
 * In EXPLORE MODE the settings are changed to all the layers. <br>
 * In SCENARIO MODE the settings are changed depending on the active layer or a layer passed to the
 * constructor of the class
 * </p>
 * <p>The properties changed in this panel are:
 * <ul>
 * <li>Scale Mode.
 * <li>Time Scale.</li>
 * <li>Visual Scale.</li>
 * </ul>
 * </p>
 * 
 * @version $Id: SettingsPanelScale.java,v 1.17 2004/05/22 13:33:38 vainio Exp $
 */
public class SettingsPanelScale extends javax.swing.JPanel {
    
    
    /**
     * Creates a new SettingsPanelScale (for EXPLORE MODE)
     * 
     * @param generalSettings
     *            instace of GeneralSettings that contains the settings to be accessed and modified
     */
    public SettingsPanelScale(GeneralSettings generalSettings) {
        this.scenarioMode = false;
        this.generalSettings = generalSettings;
        this.auxSettingsMSC = generalSettings.getSettingsMSC(generalSettings.getDefaultLayer());
        initComponents();
        this.updateValuesUI(false);
    }
    
    /**
     * Creates a new SettingsPanelScale (for SCENARIO MODE)
     * 
     * @param generalSettings
     *            instace of GeneralSettings that contains the settings to be accessed and modified
     * @param activeLayer
     *            Layer from which the settings are accessed and modified
     */
    public SettingsPanelScale(GeneralSettings generalSettings, Layer activeLayer) {
        this.scenarioMode = true;
        this.generalSettings = generalSettings;
        this.auxSettingsMSC = generalSettings.getSettingsMSC(activeLayer);
        initComponents();
        this.updateValuesUI(false);
    }

    /**
     * Creates a new SettingsPanelScale (for SCENARIO MODE)
     * 
     * @param settingsMSC
     *            instace of SettingsMSC that contains the settings to be accessed and modified
     */
    public SettingsPanelScale(SettingsMSC settingsMSC) {
        this.scenarioMode = true;
        this.auxSettingsMSC = settingsMSC;
        initComponents();
        this.updateValuesUI(false);
    }

    
    /**
     * initialize the components in the panel
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroupScaleButtons = new javax.swing.ButtonGroup();
        scaleSettingsPanel = new javax.swing.JPanel();
        jPanelScaleRadioButtons = new javax.swing.JPanel();
        labelScaleMode = new javax.swing.JLabel();
        panelRadioButtonsScale = new javax.swing.JPanel();
        radiobuttonSecs = new javax.swing.JRadioButton();
        radiobuttonDSecs = new javax.swing.JRadioButton();
        radiobuttonCSecs = new javax.swing.JRadioButton();
        radiobuttonMillisecs = new javax.swing.JRadioButton();
        labelTimeScale = new javax.swing.JLabel();
        sliderTimeScale = new javax.swing.JSlider();
        tfTimeScale = new javax.swing.JTextField();
        labelVisualScale = new javax.swing.JLabel();
        sliderVisualScale = new javax.swing.JSlider();
        tfVisualScale = new javax.swing.JTextField();

        setLayout(new java.awt.GridBagLayout());

        scaleSettingsPanel.setLayout(new java.awt.GridBagLayout());

        labelScaleMode.setText(Localization.getString("panel.settings.scale.label_scale"));
        labelScaleMode.setToolTipText(Localization.getString("panel.settings.scale.label_scale_tooltip"));
        labelScaleMode.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jPanelScaleRadioButtons.add(labelScaleMode);

        panelRadioButtonsScale.setLayout(new javax.swing.BoxLayout(panelRadioButtonsScale, javax.swing.BoxLayout.Y_AXIS));

        if (auxSettingsMSC.getScaleMode()==SettingsMSC.SCALE_MODE_SECONDS){
            radiobuttonSecs.setSelected(true);
        }
        radiobuttonSecs.setText(Localization.getString("panel.settings.scale.label_1_1"));
        radiobuttonSecs.setToolTipText(Localization.getString("panel.settings.scale.label_1_1_tooltip"));
        buttonGroupScaleButtons.add(radiobuttonSecs);
        radiobuttonSecs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radiobuttonSecsActionPerformed(evt);
            }
        });

        panelRadioButtonsScale.add(radiobuttonSecs);

        if (auxSettingsMSC.getScaleMode()==SettingsMSC.SCALE_MODE_DSECONDS){
            radiobuttonDSecs.setSelected(true);
        }
        radiobuttonDSecs.setText(Localization.getString("panel.settings.scale.label_1_0.1"));
        radiobuttonDSecs.setToolTipText(Localization.getString("panel.settings.scale.label_1_0.1_tooltip"));
        buttonGroupScaleButtons.add(radiobuttonDSecs);
        radiobuttonDSecs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radiobuttonDSecsActionPerformed(evt);
            }
        });

        panelRadioButtonsScale.add(radiobuttonDSecs);

        if (auxSettingsMSC.getScaleMode()==SettingsMSC.SCALE_MODE_CSECONDS){
            radiobuttonCSecs.setSelected(true);
        }
        radiobuttonCSecs.setText(Localization.getString("panel.settings.scale.label_1_0.01"));
        radiobuttonCSecs.setToolTipText(Localization.getString("panel.settings.scale.label_1_0.01_tooltip"));
        buttonGroupScaleButtons.add(radiobuttonCSecs);
        radiobuttonCSecs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radiobuttonCSecsActionPerformed(evt);
            }
        });

        panelRadioButtonsScale.add(radiobuttonCSecs);

        if (auxSettingsMSC.getScaleMode()==SettingsMSC.SCALE_MODE_MILLISECONDS){
            radiobuttonMillisecs.setSelected(true);
        }
        radiobuttonMillisecs.setText(Localization.getString("panel.settings.scale.label_1_0.001"));
        radiobuttonMillisecs.setToolTipText(Localization.getString("panel.settings.scale.label_1_0.001_tooltip"));
        buttonGroupScaleButtons.add(radiobuttonMillisecs);
        radiobuttonMillisecs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radiobuttonMillisecsActionPerformed(evt);
            }
        });

        panelRadioButtonsScale.add(radiobuttonMillisecs);

        jPanelScaleRadioButtons.add(panelRadioButtonsScale);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = 3;
        scaleSettingsPanel.add(jPanelScaleRadioButtons, gridBagConstraints);

        labelTimeScale.setText(Localization.getString("panel.settings.scale.label_time_scale"));
        labelTimeScale.setToolTipText(Localization.getString("panel.settings.scale.time_scale_tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        scaleSettingsPanel.add(labelTimeScale, gridBagConstraints);

        sliderTimeScale.setMaximum(1000);
        sliderTimeScale.setToolTipText(Localization.getString("panel.settings.scale.time_scale_tooltip"));
        sliderTimeScale.setValue(calculateTimeScaleSliderValue(auxSettingsMSC.getTimeScale()/auxSettingsMSC.getScaleMultiplier()));
        sliderTimeScale.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                sliderTimeScaleStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        scaleSettingsPanel.add(sliderTimeScale, gridBagConstraints);

        tfTimeScale.setColumns(10);
        tfTimeScale.setEditable(false);
        tfTimeScale.setHorizontalAlignment(javax.swing.JTextField.TRAILING);
        tfTimeScale.setToolTipText(Localization.getString("panel.settings.scale.time_scale_tooltip"));
        tfTimeScale.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tfTimeScaleActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        scaleSettingsPanel.add(tfTimeScale, gridBagConstraints);

        labelVisualScale.setText(Localization.getString("panel.settings.scale.label_visual_scale"));
        labelVisualScale.setToolTipText(Localization.getString("panel.settings.scale.visual_scale_tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        scaleSettingsPanel.add(labelVisualScale, gridBagConstraints);

        sliderVisualScale.setMajorTickSpacing(1000);
        sliderVisualScale.setMaximum(SettingsMSC.MAX_VISUAL_SCALE);
        sliderVisualScale.setMinimum(SettingsMSC.MIN_VISUAL_SCALE);
        sliderVisualScale.setToolTipText(Localization.getString("panel.settings.scale.visual_scale_tooltip"));
        sliderVisualScale.setValue((int)(auxSettingsMSC.getVisualScale()/auxSettingsMSC.getScaleMultiplier()));
        sliderVisualScale.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                sliderVisualScaleStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        scaleSettingsPanel.add(sliderVisualScale, gridBagConstraints);

        tfVisualScale.setColumns(10);
        tfVisualScale.setEditable(false);
        tfVisualScale.setHorizontalAlignment(javax.swing.JTextField.TRAILING);
        tfVisualScale.setToolTipText(Localization.getString("panel.settings.scale.visual_scale_tooltip"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        scaleSettingsPanel.add(tfVisualScale, gridBagConstraints);

        add(scaleSettingsPanel, new java.awt.GridBagConstraints());

    }//GEN-END:initComponents

    private void radiobuttonDSecsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radiobuttonDSecsActionPerformed
        changeScaleSelected(radiobuttonDSecs, SettingsMSC.SCALE_MODE_DSECONDS);
    }//GEN-LAST:event_radiobuttonDSecsActionPerformed

    private void radiobuttonCSecsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radiobuttonCSecsActionPerformed
        changeScaleSelected(radiobuttonCSecs, SettingsMSC.SCALE_MODE_CSECONDS);
    }//GEN-LAST:event_radiobuttonCSecsActionPerformed

    private void radiobuttonMillisecsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radiobuttonMillisecsActionPerformed
        changeScaleSelected(radiobuttonMillisecs, SettingsMSC.SCALE_MODE_MILLISECONDS);
    }//GEN-LAST:event_radiobuttonMillisecsActionPerformed

    private void radiobuttonSecsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radiobuttonSecsActionPerformed
        changeScaleSelected(radiobuttonSecs, SettingsMSC.SCALE_MODE_SECONDS);
    }//GEN-LAST:event_radiobuttonSecsActionPerformed
    
    private void tfTimeScaleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tfTimeScaleActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_tfTimeScaleActionPerformed
    
    private void sliderVisualScaleStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sliderVisualScaleStateChanged
        if (changeSlidersLock)
            return;
        updateValuesUI(true);
        updateValuesFromSlider();
    }//GEN-LAST:event_sliderVisualScaleStateChanged
    
    private void sliderTimeScaleStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sliderTimeScaleStateChanged
        if (changeSlidersLock)
            return;

        updateValuesUI(true);
        updateValuesFromSlider();
    }//GEN-LAST:event_sliderTimeScaleStateChanged
        
    /**
     * Calculates the value of the time scale in function of the value of the slider that represents
     * it. This method implements a logarithmic function based in the max and min time scales and
     * the max and min values assigned to the slider
     * 
     * @param x
     *            value of the slider that represents the time scale
     * @return value for the time scale
     */
    private float calculateTimeScale(int x){
        float m = (float)((Math.log(SettingsMSC.MAX_TIME_SCALE)-Math.log(SettingsMSC.MIN_TIME_SCALE))/(sliderTimeScale.getMaximum()-sliderTimeScale.getMinimum()));
        float n = (float)(Math.log(SettingsMSC.MIN_TIME_SCALE)- (m*sliderTimeScale.getMinimum()));
        Float result = new Float(Math.exp(m*x+n));
        return result.floatValue();
    }

    
    /**
     * Calculates the value that has to be set to the slider that represents the value for the time
     * scale
     * 
     * @param y
     *            current value of the time scale
     * @return value to be set to the slider that represents the range of values of the time scale.
     */
    private int calculateTimeScaleSliderValue(float y){
        float m = (float)((Math.log(SettingsMSC.MAX_TIME_SCALE)-Math.log(SettingsMSC.MIN_TIME_SCALE))/(sliderTimeScale.getMaximum()-sliderTimeScale.getMinimum()));
        float n = (float)(Math.log(SettingsMSC.MIN_TIME_SCALE)- (m*sliderTimeScale.getMinimum()));
        Float result = new Float((Math.log(y)- n)/m);
        int x = Math.round(result.floatValue());
        return x;
    }
    
    /**
     * Changes the values stored in the settings MSC to adapt them to the new scale selected
     * 
     * @param radioButton
     *            JRadioButton selected
     * @param newScaleMode
     *            int representing the scale mode to be set
     */
    private void changeScaleSelected(JRadioButton radioButton, int newScaleMode){
        int oldScale, newScale;

        if (radioButton.isSelected() && auxSettingsMSC.getScaleMode()!= newScaleMode){
            /* disable the listener methods for the sliders */
            this.changeSlidersLock = true;
            changeScale(auxSettingsMSC.getScaleMode(), newScaleMode);
            updateValuesUI(false);
            /* enable the listener methods for the sliders */
            this.changeSlidersLock = false;
        }
    }
    
    /**
     * Change the values in the SettingsMSC object to adapt them to the new scale mode selected.
     * 
     * @param oldScale int with the previous animator scale mode
     * @param newScale int with the new scale mode to set to the animator
     */
    private void changeScale (int oldScale, int newScale){

        float factor = ((float)SettingsMSC.getScaleMultiplier(newScale)/(float)SettingsMSC.getScaleMultiplier(oldScale));

        /* in explore mode all the layers are modified */ 
        if (!scenarioMode){
            for (Iterator i= generalSettings.getListSettingsMSC().iterator(); i.hasNext();){
                SettingsMSC tempSettingsMSC = (SettingsMSC)i.next();
                tempSettingsMSC.setScaleMode(newScale);
                tempSettingsMSC.setTimeScale(tempSettingsMSC.getTimeScale()*factor);
                tempSettingsMSC.setVisualScale(Math.round(tempSettingsMSC.getVisualScale()*factor));
            }
            return;
        }
        /* in scenario mode only modify the current layer */
        auxSettingsMSC.setScaleMode(newScale);
        auxSettingsMSC.setTimeScale(auxSettingsMSC.getTimeScale()*factor);
        auxSettingsMSC.setVisualScale(Math.round(auxSettingsMSC.getVisualScale()*factor));
    }
    
    
    

     /**
      * Updates the values of the elements contained in the panel to reflect the changes done in the
      * values of the settings.
      * 
      * @param fromSlider
      *            used to know if the changes made over the values of the settings come from the
      *            change of a slider value.
      */
    private void updateValuesUI(boolean fromSlider) {
        float valueTimeScale;
        int valueVisualScale;
                 
        /* Create format for displaying the decimal digits of time scale*/
        java.text.NumberFormat nf = 
            java.text.DecimalFormat.getInstance(Localization.getCurrentLocale());
        nf.setMaximumFractionDigits(3);
        nf.setMinimumFractionDigits(3);

        /* get values for displaying in the text fields from the values in the slider*/        
        if (fromSlider ==true){
            valueTimeScale = calculateTimeScale(sliderTimeScale.getValue());
            valueVisualScale = sliderVisualScale.getValue();            
        /* get values for displaying in the text fields from the values in the settings*/
        } else {
            
            valueTimeScale = auxSettingsMSC.getTimeScale()/auxSettingsMSC.getScaleMultiplier();
            valueVisualScale = auxSettingsMSC.getVisualScale()/auxSettingsMSC.getScaleMultiplier();            
            this.sliderTimeScale.setValue(calculateTimeScaleSliderValue(valueTimeScale));
            this.sliderVisualScale.setValue(valueVisualScale);
        }
        
        tfTimeScale.setText(Localization.getString("panel.settings.scale.display_time_scale", nf.format(valueTimeScale), auxSettingsMSC.getScaleMultiplierUnit())); //$NON-NLS-1$
        tfVisualScale.setText(Localization.getString("panel.settings.scale.display_visual_scale", new Integer(valueVisualScale), auxSettingsMSC.getScaleMultiplierUnit())); //$NON-NLS-1$
    }
    
    
    
    /**
     * Update the values of the settings classes from the values of the sliders that represent the
     * values selected by the user
     */
    private void updateValuesFromSlider(){
        
        float valueTimeScale;
        int valueVisualScale;
                 
        /* get values from UI */
        valueTimeScale = calculateTimeScale(sliderTimeScale.getValue());
        valueVisualScale = sliderVisualScale.getValue();
        
        /* calculate real values that the settings classes will store */
        valueTimeScale *= auxSettingsMSC.getScaleMultiplier();
        valueVisualScale *= auxSettingsMSC.getScaleMultiplier();
        
        /* if explore mode is selecte the update is done in all the layers */
        if (!scenarioMode){
            updateValuesAllLayers(auxSettingsMSC.getScaleMode(), valueVisualScale, valueTimeScale);
            return;
        }
        
        /* if the scenario mode is selected only the values for the current layer are modified */
        auxSettingsMSC.setTimeScale(valueTimeScale);
        auxSettingsMSC.setVisualScale(valueVisualScale);
    }
    
    
    /**
     * Update the values related to the scale in all the layers present in an animation. <br>
     * This feature is mainly used in EXPLORE MODE
     * 
     * @param scaleMode
     *            value to set to the scale mode
     * @param visualScale
     *            value to set to the visual scale
     * @param timeScale
     *            value to set to the time scale
     */
    private void updateValuesAllLayers(int scaleMode, int visualScale, float timeScale) {
        for (Iterator i = generalSettings.getListSettingsMSC().iterator(); i.hasNext();) {
            SettingsMSC tempSettingsMSC = (SettingsMSC) i.next();
            tempSettingsMSC.setScaleMode(scaleMode);
            tempSettingsMSC.setTimeScale(timeScale);
            tempSettingsMSC.setVisualScale(visualScale);
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupScaleButtons;
    private javax.swing.JPanel jPanelScaleRadioButtons;
    private javax.swing.JLabel labelScaleMode;
    private javax.swing.JLabel labelTimeScale;
    private javax.swing.JLabel labelVisualScale;
    private javax.swing.JPanel panelRadioButtonsScale;
    private javax.swing.JRadioButton radiobuttonCSecs;
    private javax.swing.JRadioButton radiobuttonDSecs;
    private javax.swing.JRadioButton radiobuttonMillisecs;
    private javax.swing.JRadioButton radiobuttonSecs;
    private javax.swing.JPanel scaleSettingsPanel;
    private javax.swing.JSlider sliderTimeScale;
    private javax.swing.JSlider sliderVisualScale;
    private javax.swing.JTextField tfTimeScale;
    private javax.swing.JTextField tfVisualScale;
    // End of variables declaration//GEN-END:variables
    
    /** GeneralSettings reference to access and modify the settings stored in it */
    private GeneralSettings generalSettings;
    
    /** SettingsMSC reference to access and modify the settings stored in it */
    private SettingsMSC auxSettingsMSC;

    /** true if the animator is working in SCENARIO MODE */
    private boolean scenarioMode;
    
    /**
     * avoids the execution of the sliders listeners. Those ones would call the updateValuesFrom
     * Slider, storing in the data structures wrong values
     */  
    private boolean changeSlidersLock = false;
}
