package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fi.helsinki.dacopan.Localization;

/**
 * 
 */
public class StatusBar extends JPanel {
    
    // Widgets: status area, state area and
    // a time panel
    private MainFrame mainFrame;
    
    private JLabel exploreModeLabel;
    private JLabel scenarioModeLabel;
    private JLabel statusLabel;
    private JPanel timePanel;
    private JPanel centerPanel; // This panel holds the empty status area and a place
                                // for the restore animation scenario window button
    private JButton scenarioRestoreButton;
    
	public static final Font STATUS_BAR_FONT = new Font("Verdana", Font.PLAIN, 11);
    public static final Font MODE_FONT = new Font("Verdana", Font.BOLD, 12);
    
    public static final Color MODE_COLOR = new Color(0x0066cc);
    
    public StatusBar(final MainFrame mainFrame) {
        this.mainFrame = mainFrame;
        setBorder(BorderFactory.createEmptyBorder(2,0,0,0));        
        
        // Create the panels
        JPanel statusPanel = new JPanel(new BorderLayout());
                
        statusLabel = new JLabel(" ");
        statusLabel.setBackground(SystemColor.control);
        statusLabel.setBorder(BorderFactory.createEtchedBorder());
        statusLabel.setFont(STATUS_BAR_FONT);
        centerPanel = new JPanel(new BorderLayout());  
        centerPanel.setBackground(SystemColor.control);      
        centerPanel.add(statusLabel, BorderLayout.CENTER);
        scenarioRestoreButton = new JButton(Localization.getString("animseq.dialog_title"));
        scenarioRestoreButton.setToolTipText(Localization.getString("panel.status.button_scenario_tooltip"));
        scenarioRestoreButton.setBackground(SystemColor.activeCaption);
        scenarioRestoreButton.setForeground(SystemColor.activeCaptionText);
        scenarioRestoreButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
               mainFrame.showScenarioDialog();
			}            
        });
                
        exploreModeLabel = new JLabel(Localization.getString("panel.status.mode_explore")); //$NON-NLS-1$
        exploreModeLabel.setFont(MODE_FONT);
        exploreModeLabel.setBackground(SystemColor.control);        
        exploreModeLabel.setForeground(MODE_COLOR);
        exploreModeLabel.setBorder(BorderFactory.createEtchedBorder());        
        
        scenarioModeLabel = new JLabel(Localization.getString("panel.status.mode_scenario")); //$NON-NLS-1$
        scenarioModeLabel.setFont(MODE_FONT);        
        scenarioModeLabel.setForeground(MODE_COLOR);
        scenarioModeLabel.setBackground(SystemColor.control);
        scenarioModeLabel.setBorder(BorderFactory.createEtchedBorder());        
                
        statusPanel.add(exploreModeLabel, BorderLayout.WEST);
        statusPanel.add(scenarioModeLabel, BorderLayout.EAST);        
        
        // Create an empty time panel
        timePanel = new TimePanel(null, 0f);        
        
        // Add them to the panel
    	setLayout(new BorderLayout());
        add(centerPanel, BorderLayout.CENTER);
        add(timePanel, BorderLayout.EAST);
        add(statusPanel, BorderLayout.WEST);
    }
           
    public void setTimePanel(JPanel timePanel) {
    	// Remove the old time panel before adding a new one
        remove(this.timePanel); 
        if(timePanel != null) {            
        	this.timePanel = timePanel;            
            add(this.timePanel, BorderLayout.EAST);
        }
        repaint();
    }
    
    public void setStatusText(String text) {
    	exploreModeLabel.setText(text);        
    }
    
    public void setStateMode(int mode) {
        if(mode == MainFrame.DACOPAN_EXPLORE_MODE) {
            exploreModeLabel.setEnabled(true);
            scenarioModeLabel.setEnabled(false);            
        }
        else if(mode == MainFrame.DACOPAN_SCENARIO_MODE) {
            exploreModeLabel.setEnabled(false);
            scenarioModeLabel.setEnabled(true);            
        }        
        else {
            exploreModeLabel.setEnabled(false);
            scenarioModeLabel.setEnabled(false);                        
        }
    }    
    
    public void setRestoreScenarioDialogButton(boolean mode) {
    	if(mode == true) {            
    		centerPanel.add(scenarioRestoreButton, BorderLayout.WEST);            
        }
        else {
        	centerPanel.remove(scenarioRestoreButton);
        }
    }
}
