/*
 * Created on Apr 12, 2004
 *
 * This class represents the titles in the program, so that
 * you can have a single layout/looknfeel. Just give the title text
 * as the constructor param.
 */
package fi.helsinki.dacopan.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.SystemColor;
import java.awt.geom.Rectangle2D;

import javax.swing.JPanel;

/**
 * @author jonathan
 *
 */
public class TitlePanel extends JPanel {

	public static final int FONT_WIDTH = 8;
	public static final int TITLE_HEIGHT = 20;
	public static final int SUPPOSED_MAX_LAYERNAME_CHARS = 20;
	
    private boolean isActive;
    private String titleText;
    private String layerText;
    
    /**
     * This constructs a title object (a JPanel) with
     * a title text and a layer text. The combined string
     * is represented as "titleText - layerText". Also a
     * null layerText is allowed, this just prints the
     * titleText. In both cases the panel's width is
     * calculated dynamically.
     * @param title
     * @param layerText
     */
    public TitlePanel(String title, String layerText){
        super();
        setBackground(SystemColor.activeCaption); //UserInterface.DACOPAN_COLOR_TITLE_BG);
        setForeground(SystemColor.activeCaptionText); ///UserInterface.DACOPAN_COLOR_TITLE_TEXT);
		setFont(UserInterface.DACOPAN_FONT_TITLE);
        this.titleText = title;
        this.layerText = layerText;
        Dimension goodSize = new Dimension(FONT_WIDTH*(title.length()+SUPPOSED_MAX_LAYERNAME_CHARS),
            TITLE_HEIGHT);
        setMinimumSize(goodSize);
        setPreferredSize(goodSize);
        isActive = true;
    }    


    /**
     * Draws the title
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        
        // Draw a small gradient to both the top and bottom of the panel
        Color lineColor = SystemColor.activeCaption;
        Paint linearGradient = new GradientPaint(0,-2,SystemColor.controlLtHighlight,0,4,this.getBackground());
        Paint linearGradient2 = new GradientPaint(0,getHeight()+2,SystemColor.controlLtHighlight,0,getHeight()-4,this.getBackground());

        Graphics2D g2 = (Graphics2D) g; 
        g2.setPaint(linearGradient);
        g2.fill(new Rectangle2D.Double(0.,0.,getWidth(),5.));        
        g2.setPaint(linearGradient2);
        g2.fill(new Rectangle2D.Double(0.,getHeight()-5,getWidth(),getHeight()));        
        
        g.setColor(SystemColor.controlDkShadow);
        g.drawLine(0, getHeight() -1 , getWidth() - 1, getHeight() - 1);
        
        
		String combinedTitle = titleText;
		if(layerText!=null && !"".equals(layerText)){
		 	combinedTitle += " - " + layerText;
        }
        
        if(isActive) {
            // Draw a dark shadow to the text
            g.setColor(Color.BLACK);
            g.drawString(combinedTitle,11,15);        
            
            g.setColor(SystemColor.activeCaptionText);
        }
        else {
            g.setColor(SystemColor.inactiveCaptionText);
        }
        
        // Draw the text        
        g.drawString(combinedTitle,10,14);
        
    }

    
    public void setActive(boolean value) {
        if(value == false) {
        	setBackground(SystemColor.inactiveCaption);
            isActive = false;
        }
        else {
            setBackground(SystemColor.activeCaption);
            isActive = true;
        }            
    }
    
    /**
     * @param layerText The layerText to set.
     */
    /*public void setLayerText(String layerText) {
        this.layerText = layerText;
    }*/

}
