/*
 * Created on 8.4.2004
 *  
 */
package fi.helsinki.dacopan.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JPanel;

import fi.helsinki.dacopan.Localization;
import fi.helsinki.dacopan.model.DataView;
import fi.helsinki.dacopan.model.Layer;
import fi.helsinki.dacopan.model.TransferUnit;

/**
 *  
 */
public class UFOPanel extends AbstractAnimationPanel {

    //Inner classes
    /**
     * This inner class acts as a representation of a unit which is in flight, 
     * i.e. it is being drawn in the channel area. This is to keep track of 
     * the coordinates of the unit, to detect selecting mouse clicks.
     */
    private class UnitInFlight {
 
        public TransferUnit transferUnit;
        public Point upperLeft;
        public Point lowerRight;        
        
        /**
         * Creates a unit in flight.
         */
        public UnitInFlight(TransferUnit transferUnit, Point upperLeft, Point lowerRight) {
            this.transferUnit = transferUnit;
            this.upperLeft = upperLeft;
            this.lowerRight = lowerRight;
        }

        public boolean isInside(int x, int y) {
            if (x > upperLeft.x && x < lowerRight.x && y > upperLeft.y && y < lowerRight.y) {
                return true;
            } else {
                return false;
            }
        }
    }



    // Constants
    public static final int L_TO_R = 0; //used to index the array of currently active units
    public static final int R_TO_L = 1; //used to index the array of currently active units
    public static final int PLAY_MODE = 0;
    public static final int PAUSE_MODE = ((0x23 << 2) / 10 - 015); //Are you good at math?

    // Global variables for UFO panel
    private int currentMode;
    private Collection inFlightUnits;
    private final DataView dataView;
    private final Layer layer;

    // Global widgets
    private GridBagLayout gridBag;
    private TitlePanel ufoTitle;
    private ChannelPanel channelPanel;
    private JButton leftShowEncButton;
    private JButton rightShowEncButton;
    
    private boolean isLeftButtonEnabled;
    private boolean isRightButtonEnabled;
    
    public static final Color DACOPAN_UFO_UNIT_COLOR_1 = Color.LIGHT_GRAY;
    public static final Color DACOPAN_UFO_UNIT_COLOR_2 = Color.ORANGE;
    public static final int DACOPAN_UFO_UNIT_SIZE = 50;

    /**
     * @param mainFrame A reference to the MainFrame
     * @param data      A reference to the DataView
     */
    public UFOPanel(MainFrame mainFrame, DataView data, Layer layer) {
        super(mainFrame);
        
        this.dataView = data;
        this.layer = layer;
        Dimension d = new Dimension();
        d.setSize(200,300);
        this.setMinimumSize(d);

        setLayout(new BorderLayout());
               
        inFlightUnits = new ArrayList(); //initialize
        addWidgets();
    }
    
    public UFOPanel(MainFrame mainFrame, Layer layer)
    {
        this(mainFrame, mainFrame.getDataView(), layer);
    }

    /**
     * Creates all the widgets used in the UFO panel and adds them to it in correct places.
     */
    protected void addWidgets() {

        //Create widgets
        if(layer!=null){
            ufoTitle = new TitlePanel(Localization.getString("panel.ufo.title"), layer.getName()); //$NON-NLS-1$
            ufoTitle.setActive(true);
        }else{
            ufoTitle = new TitlePanel(Localization.getString("panel.ufo.title"), null); //$NON-NLS-1$
            ufoTitle.setActive(false);
        }
        
        leftShowEncButton = new JButton(Localization.getString("panel.ufo.show_enc")); //$NON-NLS-1$
        UserInterface.setDefaultButtonFont(leftShowEncButton);        
        leftShowEncButton.setToolTipText(Localization.getString("panel.ufo.show_enc_tooltip")); //$NON-NLS-1$
        leftShowEncButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // Call the MainFrame and ask it to change to ENC mode
                mainFrame.setupAnimationModeENC(channelPanel.getCurrentlyActive(ChannelPanel.LEFT_TO_RIGHT));
            }
        });
        leftShowEncButton.setEnabled(false);
        isLeftButtonEnabled = false;
        
        rightShowEncButton = new JButton(Localization.getString("panel.ufo.show_enc")); //$NON-NLS-1$
        UserInterface.setDefaultButtonFont(rightShowEncButton);        
        rightShowEncButton.setToolTipText(Localization.getString("panel.ufo.show_enc_tooltip")); //$NON-NLS-1$
        rightShowEncButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // Call the MainFrame and ask it to change to ENC mode            	
                mainFrame.setupAnimationModeENC(channelPanel.getCurrentlyActive(ChannelPanel.RIGHT_TO_LEFT));
            }
        });
        rightShowEncButton.setEnabled(false);
        isRightButtonEnabled = false;
                       
        /*
         * Create the channel panel and set up a mouse listener for it to handle 
         * clicking the units in order to activate them.
         */
        channelPanel = new ChannelPanel(dataView.getHosts());
        MouseListener ml = new MouseAdapter() {
            /**
             * Handles the mouse clicked event by checking if the mouse coordinates 
             * are inside some unit in flight.
             * 
             * @param me The mouse event
             */
            public void mouseClicked(MouseEvent me) {
                for (Iterator iter = inFlightUnits.iterator(); iter.hasNext();) {
                    UnitInFlight uif = (UnitInFlight) iter.next();
                    if (uif.isInside(me.getX(), me.getY())) {

                    }
                }
            }
        };
        channelPanel.addMouseListener(ml);

        // Create a layout for widgets
        add(ufoTitle, BorderLayout.NORTH);

        add(channelPanel, BorderLayout.CENTER);

        JPanel southPanel = new JPanel(new GridBagLayout());
        southPanel.setBackground(Color.WHITE);
        GridBagConstraints c = new GridBagConstraints();
        
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5,5,5,5);       
        southPanel.add(leftShowEncButton, c);

        c.gridx = 1;
        c.gridy = 0;        
        southPanel.add(rightShowEncButton, c);
        
        add(southPanel, BorderLayout.SOUTH);
        
        //and here we detect wether the UFOpanel is disabled or not
        //if layer == null, set all widgets to disable
        if(layer==null){
            leftShowEncButton.setEnabled(false);
            rightShowEncButton.setEnabled(false);
        }        
    }

    /**
     * @param nowTime
     */
    public void stepTo(float nowTime) {
        List units = dataView.getUnitsForLayer(layer, nowTime, nowTime);
        //channelPanel.drawUnits(units, nowTime, mainFrame.getSettings().getAntiAliasing());
        // antialiasing disabled permanently for efficiency reasons
        channelPanel.drawUnits(units, nowTime, false);
        
        // Disable the buttons that should not be pressed
        updateButtons();
    }

    /**
     * Updates the state of the show encapsulation
     * buttons based on the values of the currently
     * selected units in the channel panel.
     */
    public void updateButtons() {
        boolean newLeftEnabled;
        boolean newRightEnabled;
        
        if (channelPanel.getCurrentlyActive(ChannelPanel.LEFT_TO_RIGHT) == null) {
            newLeftEnabled = false;
        } else {
            newLeftEnabled = true;
        }
        
        if (channelPanel.getCurrentlyActive(ChannelPanel.RIGHT_TO_LEFT) == null) {
            newRightEnabled = false;
        } else {
            newRightEnabled = true;
        }
        
        if (newRightEnabled != this.isRightButtonEnabled) {
            this.isRightButtonEnabled = newRightEnabled;
            rightShowEncButton.setEnabled(newRightEnabled);
        }
        
        if (newLeftEnabled != this.isLeftButtonEnabled) {
            this.isLeftButtonEnabled = newLeftEnabled;
            leftShowEncButton.setEnabled(newLeftEnabled);
        }
    }
}
