package fi.helsinki.dacopan.model;

import fi.helsinki.dacopan.AbstractTestCase;

/**
 * Test case for the step iterator used to step thru animation sequences.
 * 
 * @version $Id: ScenarioStepIteratorTestCase.java,v 1.5 2004/05/27 07:27:10 jaarnial Exp $
 */
public class ScenarioStepIteratorTestCase extends AbstractTestCase {
    
    private ScenarioStepIterator it;
    private DataView view = new DefaultDataView();;
    private NoteManager noteManager = new NoteManager();

    protected void setUp() throws Exception {
        view.addLayer(NETWORK);
        view.addProtocol(IPv4);
        view.addTransferUnit(createUnit(0.1f, 0.4f), false);
        view.addTransferUnit(createUnit(0.3f, 0.5f), false);
        view.addTransferUnit(createUnit(0.8f, 1.0f), false);
        view.addTransferUnit(createUnit(0.9f, 1.4f), false);
        view.addTransferUnit(createUnit(1.3f, 1.8f), false);
        view.addTransferUnit(createUnit(1.4f, 2.2f), false);
        // 0.1 0.3 0.4 0.5 0.8 0.9 1.0 1.3 1.4 1.8 2.2
        it = new ScenarioStepIterator(NETWORK, view, noteManager);
    }
    
    public void testStepWithoutNotes() {
        assertFalse(it.hasPrevious());
        assertEquals(0.0f, it.current(), 0.0f);
        assertEquals(0.1f, it.next(), 0.0f);
        assertEquals(0.3f, it.next(), 0.0f);
        assertEquals(0.4f, it.next(), 0.0f);
        assertEquals(1.8f, it.getNextForTime(1.5f), 0.0f);
    }
    
    public void testStepWithNotes() {
        Note a = new Note(NETWORK, 0.2f, "Test");
        Note b = new Note(NETWORK, 0.9f, "Test");
        Note c = new Note(NETWORK, 1.1f, "Test");
        Note d = new Note(NETWORK, 0.95f, "Test");
        Note e = new Note(NETWORK, 1.05f, "Test");
        noteManager.addNoteTimeLayer(a);
        noteManager.addNoteTimeLayer(b);
        noteManager.addNoteTimeLayer(c);
        noteManager.addNoteTimeLayer(d);
        noteManager.addNoteTimeLayer(e);
        // 0.1 0.2 0.3 0.4 0.5 0.8 0.9 0.95 1.0 1.05 1.1 1.3 1.4 1.8 2.2
        assertEquals(0.1f, it.next(), 0.0f);
        assertEquals(0.2f, it.next(), 0.0f);
        assertEquals(1.1f, it.getNextForTime(1.05f), 0.0f);
        assertEquals(1.3f, it.next(), 0.0f);
        assertEquals(1.1f, it.getPreviousForTime(1.2f), 0.0f);
        assertEquals(1.0f, it.getPreviousForTime(1.05f), 0.0f);
        assertEquals(0.8f, it.getPreviousForTime(0.9f), 0.0f);
        assertEquals(2.2f, it.getNextForTime(2.4f), 0.0f);
        assertEquals(1.4f, it.getNextForTime(1.3f), 0.0f);
        assertEquals(1.3f, it.previous(), 0.0f);
        assertEquals(0.9f, it.getNextForTime(0.8f), 0.0f);
        assertEquals(0.95f, it.next(), 0.0f);
        assertEquals(1.1f, it.getNextForTime(1.05f), 0.0f);
        assertEquals(1.05f, it.previous(), 0.0f);
    }
    
    public void testRemoveNotesWhileStepping() {
        Note a = new Note(NETWORK, 0.2f, "Test");
        Note b = new Note(NETWORK, 0.9f, "Test");
        Note c = new Note(NETWORK, 1.1f, "Test");
        noteManager.addNoteTimeLayer(a);
        noteManager.addNoteTimeLayer(b);
        noteManager.addNoteTimeLayer(c);
        // 0.1 0.2 0.3 0.4 0.5 0.8 0.9 1.0 1.1 1.3 1.4 1.8 2.2
        assertEquals(1.0f, it.getNextForTime(0.9f), 0.0f);
        assertEquals(1.1f, it.next(), 0.0f);
        noteManager.deleteNote(c);
        assertEquals(1.0f, it.previous(), 0.0f);
        assertEquals(1.3f, it.next(), 0.0f);
        assertEquals(0.3f, it.getPreviousForTime(0.4f), 0.0f);
        noteManager.deleteNote(a);
        assertEquals(0.1f, it.previous(), 0.0f);
    }
    
    public void testAddNotesWhileStepping() {
        Note a = new Note(NETWORK, 0.2f, "Test");
        Note c = new Note(NETWORK, 1.1f, "Test");
        // 0.1 0.3 0.4 0.5 0.8 0.9 1.0 1.3 1.4 1.8 2.2
        assertEquals(1.3f, it.getNextForTime(1.0f), 0.0f);
        noteManager.addNoteTimeLayer(c);
        assertEquals(1.1f, it.getNextForTime(1.0f), 0.0f);
        assertEquals(0.1f, it.getPreviousForTime(0.3f), 0.0f);
        noteManager.addNoteTimeLayer(a);
        assertEquals(0.2f, it.getPreviousForTime(0.3f), 0.0f);
    }

}
