package fi.helsinki.dacopan.ui;

import java.io.File;

import fi.helsinki.dacopan.AbstractTestCase;
import fi.helsinki.dacopan.model.NoteManager;

/**
 * Test cases for the main frame: Test all the parts of the 
 * main frame that aren't directly related to drawing the UI. 
 */
public class MainFrameTestCase extends AbstractTestCase {
    
    public void testSetAnimatorWorkingMode() {
    	MainFrame mainFrame = new MainFrame(getRandomData(2,8,1,0.2f,true), new NoteManager());
        
        // At the beginning the animator mode should be DACOPAN_EXPLORE_MODE 
        // (as the main frame is created with random data)
        assertTrue(mainFrame.isInExploreMode());
        
        // Set the mode to explore mode
        mainFrame.setAnimatorMode(MainFrame.DACOPAN_EXPLORE_MODE);
        assertTrue(mainFrame.isInExploreMode());
        assertFalse(mainFrame.isInScenarioMode());

        // Set the mode to scenario mode
        mainFrame.setAnimatorMode(MainFrame.DACOPAN_SCENARIO_MODE);
        assertTrue(mainFrame.isInScenarioMode());
        assertFalse(mainFrame.isInExploreMode());
        
        // Set the mode to "no mode"
        mainFrame.setAnimatorMode(MainFrame.DACOPAN_NO_MODE);
        assertFalse(mainFrame.isInScenarioMode());
        assertFalse(mainFrame.isInExploreMode());
        
        mainFrame.dispose();
    }
    
    
    public void testLoadAnimationFile() {
    	MainFrame mainFrame = new MainFrame();      
        
        // Try to load a file that doesn't exist
        mainFrame.loadAnimationFile(new File("jou123"));
        
        FileInformation fileInfo = mainFrame.getFileInformation();
        assertFalse(fileInfo.isLoaded());
        assertFalse(fileInfo.isScenarioFile());
        assertFalse(fileInfo.isModified());
        assertNull(fileInfo.getFile());
   
        // Try to load a pef file successfully (this test is broken..)
        /*
        File testFile = new File(getClass().getResource("a.pef").getPath());
        mainFrame.loadAnimationFile();
                       
        FileInformation fileInfo = mainFrame.getFileInformation();
        assertEquals(true, fileInfo.isLoaded());
        assertEquals(false, fileInfo.isScenarioFile());
        assertEquals(false, fileInfo.isModified());
        assertEquals(testFile, fileInfo.getFile());*/

        // After that, try to load an unsuccesfull file again - the old
        // file should remain loaded
        /*mainFrame.loadAnimationFile(new File("jou123"));
        
        FileInformation fileInfo = mainFrame.getFileInformation();
        assertEquals(true, fileInfo.isLoaded());
        assertEquals(false, fileInfo.isScenarioFile());
        assertEquals(false, fileInfo.isModified());
        assertEquals(testFile, fileInfo.getFile());*/                
        
        mainFrame.dispose();
    }
    
    
    public void testSetupMSC() {
        
    }
    
    
    public void testSetupENC() {
        
    }
}
